@@ -43,7 +43,7 @@ def rpc_export(rpc_method_name, sync=False):
 
 
 def command(name, nargs=0, complete=None, range=None, count=None, bang=False,
-            register=False, sync=False, eval=None):
+            register=False, sync=False, allow_nested=False, eval=None):
     """Tag a function or plugin method as a Nvim command handler."""
     def dec(f):
         f._nvim_rpc_method_name = 'command:{}'.format(name)
@@ -73,17 +73,22 @@ def command(name, nargs=0, complete=None, range=None, count=None, bang=False,
         if eval:
             opts['eval'] = eval
 
+        if not sync and allow_nested:
+            rpc_sync = "urgent"
+        else:
+            rpc_sync = sync
+
         f._nvim_rpc_spec = {
             'type': 'command',
             'name': name,
-            'sync': sync,
+            'sync': rpc_sync,
             'opts': opts
         }
         return f
     return dec
 
 
-def autocmd(name, pattern='*', sync=False, eval=None):
+def autocmd(name, pattern='*', sync=False, allow_nested=False, eval=None):
     """Tag a function or plugin method as a Nvim autocommand handler."""
     def dec(f):
         f._nvim_rpc_method_name = 'autocmd:{}:{}'.format(name, pattern)
@@ -98,17 +103,22 @@ def autocmd(name, pattern='*', sync=False, eval=None):
         if eval:
             opts['eval'] = eval
 
+        if not sync and allow_nested:
+            rpc_sync = "urgent"
+        else:
+            rpc_sync = sync
+
         f._nvim_rpc_spec = {
             'type': 'autocmd',
             'name': name,
-            'sync': sync,
+            'sync': rpc_sync,
             'opts': opts
         }
         return f
     return dec
 
 
-def function(name, range=False, sync=False, eval=None):
+def function(name, range=False, sync=False, allow_nested=False, eval=None):
     """Tag a function or plugin method as a Nvim function handler."""
     def dec(f):
         f._nvim_rpc_method_name = 'function:{}'.format(name)
@@ -124,10 +134,15 @@ def function(name, range=False, sync=False, eval=None):
         if eval:
             opts['eval'] = eval
 
+        if not sync and allow_nested:
+            rpc_sync = "urgent"
+        else:
+            rpc_sync = sync
+
         f._nvim_rpc_spec = {
             'type': 'function',
             'name': name,
-            'sync': sync,
+            'sync': rpc_sync,
             'opts': opts
         }
         return f
